@@ -193,6 +193,34 @@ test_mcmc(model = code, name = 'block pmcmc', inits = inits, data = c(testdata,
   resultsTolerance = list(mean = list(sigma_x = .5,
                                       sigma_y = .5)))
 
+
+test_that("user defined pf algo works in PMCMC", {
+  
+  userDefinedPF <- nimbleFunction( 
+    setup = function(model, nodes, control = list()){
+      dims <- lapply(nodes, function(n) nimDim(model[[n]]))
+      mvEWSpec <- modelValuesConf(vars = c('x'), types = c('double'),
+                                  sizes = list(x = dims[[1]]))
+      mvEWSamples <- modelValues(mvEWSpec)
+    },
+    run = function(m = integer()){
+      returnType(double(0))
+      resize(mvEWSamples, m)
+      return(-Inf)
+    }
+  )
+  
+  testModel <- nimbleModel(code = code, constants = consts, data = testdata, inits = inits)
+  testModel$simulate()
+  testMCMCconf <- configureMCMC(testModel, nodes = NULL)
+  testMCMCconf$addSampler(target = c('sigma_x', 'sigma_y'), type = 'RW_PF_block', control = list(latents = 'x',
+                                                                                                 pfType = userDefinedPF))
+  testMCMC <- buildMCMC(testMCMCconf)
+  testMCMC$run(100)
+  ctestModel <- compileNimble(testModel)
+  ctestMCMC <- compileNimble(testMCMC, project = testModel)
+  ctestMCMC$run(100)
+})
 ## Let's stop here to save testing time
 ## # test MCMC with longer runs and lower tolerance
 ## set.seed(0)
@@ -230,3 +258,17 @@ test_mcmc(model = code, name = 'block pmcmc', inits = inits, data = c(testdata,
 ##                 sigma_y = sigma_y)),
 ##   resultsTolerance = list(mean = list(sigma_x = .1,
 ##                                       sigma_y = .1)))
+context("Testing of different resampling algorithms")
+set.seed(1)
+n <- 1000
+wtsList <- list(
+  rep(1/n, n),
+  c(1, rep(0, n-1)),
+  rep(1, n),
+  rdirch(1, rep(1, n)),
+  1
+)
+test_resampler('systematicResampleFunction', wtsList)
+test_resampler('stratifiedResampleFunction', wtsList)
+test_resampler('residualResampleFunction', wtsList)
+test_resampler('multinomialResampleFunction', wtsList)
